'use strict';
// if (typeof window === 'undefined' && typeof require === 'function') {
//   var fetch = require('node-fetch')
// }
const ls = {
  get(k) {
    let v = localStorage.getItem(k)
    if (!v) {
      return undefined
    }
    const t = v.split(':-')[0]
    v = v.split(':-')[1]
    switch (t) {
      case 'string':
        return v
        break;
      case 'number':
        return parseFloat(v)
        break;
      case 'object':
        return JSON.parse(v)
        break;
      case 'boolean':
        return v === 'false' ? false : true
        break;
      default:
        return v
        break;
    }
  },
  set(k, v) {
    const t = typeof v
    if (t === 'object') {
      v = JSON.stringify(v)
    }
    localStorage.setItem(k, t + ':-' + v)
  }
}

class Api {
  constructor(...rest) { // rest=[host='http://',eache=false];
    switch (rest.length) {
      case 1:
        this.host = rest[0]
        break;
      case 2:
        if (typeof rest[0] == 'string' && typeof rest[1] == 'boolean') {
          this.host = rest[0]
          this.eache = rest[1]
        } else if (typeof rest[0] == 'string' && typeof rest[1] == 'object') {
          this.host = rest[0]
          this.eache = rest[1].eache
          this.headers = rest[1].headers
        }
        break;
      default:
        break;
    }
  }
  get = (url, params, option) => {
    let eache = this.eache
    let headers = this.headers
    if (option) {
      eache = option.eache
      headers = option.headers
    }
    url = this.host + url
    if (params) {
      const paramsArray = [];
      // 拼接参数
      Object.keys(params).forEach(key => paramsArray.push(key + '=' + params[key]));
      if (url.search(/\?/) === -1) {
        url += '?' + paramsArray.join('&');
      } else {
        url += '&' + paramsArray.join('&');
      }
    }
    // 检查缓存
    if (eache) {
      const localEache = ls.get(`api-get:${JSON.stringify(params)}` + url)
      if (localEache) {
        return localEache
      }
    }
    // fetch请求
    const myFetch = fetch(url, {
      method: 'GET',
      credentials: 'include',
      headers: {
        ...headers
      },
    })
    return new Promise((resolve, reject) => {
      this.timeOut(myFetch)
        .then(response => {
          return response.json()
        })
        .then(res => {
          resolve(res)
          if (eache) {
            ls.set('api:' + url, res)
          }
        })
        .catch(e => {
          reject(e);
        });
    })
  }
  post = (url, body, option) => {
    let eache = this.eache
    let headers = this.headers
    if (option) {
      eache = option.eache
      headers = option.headers
    }
    url = this.host + url
    // Default options are marked with *
    // 检查缓存
    if (eache) {
      const localEache = ls.get(`api-post:${JSON.stringify(params)}` + url)
      if (localEache) {
        return localEache
      }
    }
    const myFetch = fetch(url, {
      body: JSON.stringify(body), // must match 'Content-Type' header
      cache: 'no-cache', // *default, no-cache, reload, force-cache, only-if-cached
      credentials: 'include', // include, same-origin, *omit
      headers: {
        'content-type': 'application/json',
        ...headers
      },
      method: 'POST', // *GET, POST, PUT, DELETE, etc. 88019005
      mode: 'cors', // no-cors, cors, *same-origin
      redirect: 'follow', // manual, *follow, error
      referrer: 'no-referrer' // *client, no-referrer
    })

    return new Promise((resolve, reject) => {
      this.timeOut(myFetch)
        .then(response => {
          return response.json();
        })
        .then(res => {
          resolve(res)
          if (eache) {
            ls.set('api:' + url, res)
          }
        })
        .catch(e => {
          reject(e);
        });
    })
  }
  put = (url, body, option) => {
    let eache = this.eache
    let headers = this.headers
    if (option) {
      eache = option.eache
      headers = option.headers
    }
    url = this.host + url
    // Default options are marked with *
    // 检查缓存
    if (eache) {
      const localEache = ls.get(`api-put:${JSON.stringify(params)}` + url)
      if (localEache) {
        return localEache
      }
    }
    const myFetch = fetch(url, {
      body: JSON.stringify(body), // must match 'Content-Type' header
      cache: 'no-cache', // *default, no-cache, reload, force-cache, only-if-cached
      credentials: 'include', // include, same-origin, *omit
      headers: {
        'content-type': 'application/json',
        ...headers
      },
      method: 'PUT', // *GET, POST, PUT, DELETE, etc. 88019005
      mode: 'cors', // no-cors, cors, *same-origin
      redirect: 'follow', // manual, *follow, error
      referrer: 'no-referrer' // *client, no-referrer
    })

    return new Promise((resolve, reject) => {
      this.timeOut(myFetch)
        .then(response => {
          return response.json();
        })
        .then(res => {
          resolve(res)
          if (eache) {
            ls.set('api:' + url, res)
          }
        })
        .catch(e => {
          reject(e);
        });
    })
  }
  patch = (url, body, option) => {
    let eache = this.eache
    let headers = this.headers
    if (option) {
      eache = option.eache
      headers = option.headers
    }
    url = this.host + url
    // Default options are marked with *
    // 检查缓存
    if (eache) {
      const localEache = ls.get(`api-patch:${JSON.stringify(params)}` + url)
      if (localEache) {
        return localEache
      }
    }
    const myFetch = fetch(url, {
      body: JSON.stringify(body), // must match 'Content-Type' header
      cache: 'no-cache', // *default, no-cache, reload, force-cache, only-if-cached
      credentials: 'include', // include, same-origin, *omit
      headers: {
        'content-type': 'application/json',
        ...headers
      },
      method: 'PATCH', // *GET, POST, PUT, DELETE, etc. 88019005
      mode: 'cors', // no-cors, cors, *same-origin
      redirect: 'follow', // manual, *follow, error
      referrer: 'no-referrer' // *client, no-referrer
    })

    return new Promise((resolve, reject) => {
      this.timeOut(myFetch)
        .then(response => {
          return response.json();
        })
        .then(res => {
          resolve(res)
          if (eache) {
            ls.set('api:' + url, res)
          }
        })
        .catch(e => {
          reject(e);
        });
    })
  }
  del = (url, body, option) => {
    let eache = this.eache
    let headers = this.headers
    if (option) {
      eache = option.eache
      headers = option.headers
    }
    url = this.host + url
    // Default options are marked with *
    // 检查缓存
    if (eache) {
      const localEache = ls.get(`api-delete:${JSON.stringify(params)}` + url)
      if (localEache) {
        return localEache
      }
    }
    const myFetch = fetch(url, {
      body: JSON.stringify(body), // must match 'Content-Type' header
      cache: 'no-cache', // *default, no-cache, reload, force-cache, only-if-cached
      credentials: 'include', // include, same-origin, *omit
      headers: {
        'content-type': 'application/json',
        ...headers
      },
      method: 'DELETE', // *GET, POST, PUT, DELETE, etc. 88019005
      mode: 'cors', // no-cors, cors, *same-origin
      redirect: 'follow', // manual, *follow, error
      referrer: 'no-referrer' // *client, no-referrer
    })

    return new Promise((resolve, reject) => {
      this.timeOut(myFetch)
        .then(response => {
          return response.json();
        })
        .then(res => {
          resolve(res)
          if (eache) {
            ls.set('api:' + url, res)
          }
        })
        .catch(e => {
          reject(e);
        });
    })
  }
  timeOut(requestPromise, timeout = 10000) {
    let timeoutAction = null;
    const timerPromise = new Promise((resolve, reject) => {
      timeoutAction = () => {
        reject('请求超时');
      }
    })
    setTimeout(() => {
      timeoutAction()
    }, timeout)
    return Promise.race([requestPromise, timerPromise]);
  }
}

module.exports = Api
